/*
Robot evitaostacoli
Sistema di base per gestire due servomotori e un sensore ultrasuoni.
Parte del corso di robotica CodemotionKids 2015 
http://www.codemotionkids.com/

Autore: Paolo De Gasperis
paolo@discienza.org


*/


#include <Servo.h> // carico la libreria Servo che contiene le istruzioni per Arduino necessarie a gestire i servomotori

// Creo i due motori con la funzione Servo
Servo motoreSinistro;  
Servo motoreDestro;


// specifica i pin di trig e echo usati per il sensore ad ultrasuoni
const int TrigPin = 8; // allaccio sul pin 8 della Arduino il pin che attiva lo sparo del sensore ultrasuoni
const int EchoPin = 9; // allaccio al pin 9 della Arduino il pin che legge il rilevatore del segnale ultrasuoni inviato
unsigned int distanza;               // distanza rilevata dal sensore unsigned significa che non avrà mai valori negativi. Il sensore a volte ci resitutisce valori negativi quando non ha nulla davanti
int durata;                 // durata del segnale di ritorno rilevato dal sensore a ultrasuoni


void setup()
{
  Serial.begin(9600);     // Inizializza la comunicazione con la seriale a 9600 bit per secondo
  pinMode(TrigPin, OUTPUT);  // Il pin allacciato al Trig del sensore spara il segnale quindi è un OUTPUT
  pinMode(EchoPin, INPUT);   // Il pin allacciato al pin Echo del sensore riceve il segnale di ritorno quindi è un INPUT
  motoreSinistro.attach(11); // attacco il motore sinistro al pin 11
  motoreDestro.attach(10);  // attacco il motore destro al pin 10
}


void loop() // il loop è il ciclo di azioni che faranno le componenti del nostro robot
{
   {
    funzioneultrasuoni(); // esegue nel loop tutto quello scritto nella funzione-ultrasuoni


  }
  
}


void funzioneultrasuoni()      // Questa è la funzione che gestisce il sensore ultrasuoni e che fa cambiare marcia ai motori se il sesore rileva un ostacolo
{
  digitalWrite(TrigPin, LOW);   // Sparo del segnale spento
  delayMicroseconds(2);          // aspetto 2 microsecondi
  digitalWrite(TrigPin, HIGH);  // accendo lo sparo del segnale, ora il sensore sta inviando ultrasuoni
  delayMicroseconds(10);         // aspetto 10 microsecondi 
  digitalWrite(TrigPin, LOW);  // spengo l'invio di ultrasuoni

  durata = pulseIn(EchoPin, HIGH);  // accendo il rilevatore del sensore e chiedo quando tempo ha impiegato il segnale a tornare
  distanza = (durata/2)/29;         // il valore inviato è la distanza che divido per 2 (andata e ritorno) e poi per 29 che è frutto della conversione della velocità del suono in cm/s diviso per due

  if (distanza < 15) {  // se la distanza è minore di 15cm
    motoreDestro.write(0);  // allora inverto la rotta del motore destro
    motoreSinistro.write(0); // il motore sinistro rimane nella sua direzione così il robot gira su se stesso
    // i motori hanno valori uguali perchè girano nella stessa direzione ma poichè sono montati uno di fronte l'altro l'effetto è quello di girare in direzioni opposte
    Serial.print("giro"); // faccio scrivere la parola giro così dal seriale capisco che sta girando
  }

  else {                   // altrimenti (quindi se non è inferiore a 15cm)
    motoreDestro.write(180);  // il motore destro gira in una direzione
    motoreSinistro.write(0);  // il motore sinistro gira in quella opposta ma poichè sono messi di fronte girano nella stessa direzione
    Serial.print("vado");    // faccio scrivere la parola vado così dal seriale capisco che sta andando avanti
  }
  if (distanza >= 200 ){  // se la distanza poi è maggiore o uguale a 200cm (2metri) significa che non ho un oggetto di fronte fino a 2metri
    Serial.println("Campo libero!"); // scrivo sul monitor un messaggio che mi indica che non ho nulla davanti

  }
  else {               // altrimenti, significa che c'è qualcosa di fronte a meno di 2metri
    Serial.print("distanza: ");    // faccio scrivere la parola distanza
    Serial.print(distanza);    // mi faccio indicare la distanza
    Serial.print("cm: ");       // faccio scrivere la parola cm
    Serial.println();          // vado una riga sotto

  }
  delay(500); // aspetto 500 millisecondi prima di riniziare
}




